package com.yanzuoguang.util.helper;


import com.yanzuoguang.util.exception.CodeException;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.HttpURLConnection;

/**
 * HTTP请求工具类
 */
public class HttpHelper {

    /**
     * 发送POST请求，当请求失败时，抛出异常或返回空字符串
     *
     * @param url        目的地址
     * @param jsonString 请求参数，json字符串。
     * @return 远程响应结果
     */
    public static String post(String url, String jsonString) {
        try {
            // 打开URL连接
            java.net.HttpURLConnection httpConn = getConn(url);
            return post(httpConn, url, jsonString);
        } catch (Exception ex) {
            throw new CodeException(ex);
        }
    }

    /**
     * 发送POST请求，当请求失败时，抛出异常或返回空字符串
     *
     * @param url        目的地址
     * @param jsonString 请求参数，json字符串。
     * @return 远程响应结果
     */
    public static String postApplicationJSON(String url, String jsonString) {
        try {
            // 打开URL连接
            java.net.HttpURLConnection httpConn = getConn(url);
            httpConn.setRequestProperty("Content-Type", "application/json");
            return post(httpConn, url, jsonString);
        } catch (Exception ex) {
            throw new CodeException(ex);
        }
    }

    /**
     * 获取连接
     *
     * @param url
     * @return
     * @throws IOException
     */
    private static HttpURLConnection getConn(String url) throws IOException {
        // 创建URL对象
        java.net.URL connURL = new java.net.URL(url);
        // 打开URL连接
        java.net.HttpURLConnection httpConn = (java.net.HttpURLConnection) connURL.openConnection();
        // 设置通用属性
        httpConn.setRequestProperty("Accept", "*/*");
        httpConn.setRequestProperty("Connection", "Keep-Alive");
        httpConn.setRequestProperty("User-Agent",
                "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1)");
        // 设置POST方式
        httpConn.setDoInput(true);
        httpConn.setDoOutput(true);
        return httpConn;
    }


    /**
     * 发送POST请求，当请求失败时，抛出异常或返回空字符串
     *
     * @param url        目的地址
     * @param jsonString 请求参数，json字符串。
     * @return 远程响应结果
     */
    public static String post(HttpURLConnection httpConn, String url, String jsonString) throws IOException {
        String result = "";// 返回的结果
        BufferedReader in = null;// 读取响应输入流
        PrintWriter out = null;
        StringBuffer sb = new StringBuffer();// 处理请求参数
        String params = "";
        try {
            params = jsonString;
            // 获取HttpURLConnection对象对应的输出流
            out = new PrintWriter(httpConn.getOutputStream());
            // 发送请求参数
            out.write(params);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应，设置编码方式
            in = new BufferedReader(new InputStreamReader(httpConn
                    .getInputStream(), "UTF-8"));
            String line;
            // 读取返回的内容
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } finally {
            if (out != null) {
                out.close();
            }
            if (in != null) {
                in.close();
            }
        }
        return result;
    }
}
