package com.yanzuoguang.util.thread;

/**
 * 多线程处理时用于线程数量控制
 */
public class ThreadListNum {

    /**
     * 最大数量
     */
    private volatile int rowTotal;

    /**
     * 执行数量
     */
    private volatile int rowExecute;

    /**
     * 执行错误数量
     */
    private volatile int rowError;

    public int getRowTotal() {
        return rowTotal;
    }

    public void setRowTotal(int rowTotal) {
        this.rowTotal = rowTotal;
    }

    public int getRowExecute() {
        return rowExecute;
    }

    public void setRowExecute(int rowExecute) {
        this.rowExecute = rowExecute;
    }

    public int getRowError() {
        return rowError;
    }

    public void setRowError(int rowError) {
        this.rowError = rowError;
    }

    /**
     * 初始化总数量
     *
     * @param total
     */
    public void init(int total) {
        synchronized (this) {
            // 初始化任务数量
            if (rowExecute == rowTotal) {
                rowTotal = rowError = rowExecute = 0;
            }
            rowTotal += total;
        }
    }

    /**
     * 判断是否等待线程
     *
     * @return
     */
    public boolean isWait() {
        synchronized (this) {
            return rowExecute < rowTotal;
        }
    }

    /**
     * 添加成功数量
     *
     * @return
     */
    public void addSuccess() {
        synchronized (this) {
            rowExecute++;
        }
    }

    /**
     * 添加失败数量
     */
    public void addError() {
        synchronized (this) {
            rowExecute++;
            rowError++;
        }
    }

    /**
     * 清空所有的数据
     */
    public void clear() {
        synchronized (this) {
            rowTotal = rowError = rowExecute = 0;
        }
    }
}
